@@ -28,15 +28,17 @@
 #include <haproxy/list.h>
 #include <haproxy/sample.h>
 
+struct resolv_requester;
+struct dns_counters;
+
 int act_resolution_cb(struct resolv_requester *requester, struct dns_counters *counters);
 int act_resolution_error_cb(struct resolv_requester *requester, int error_code);
 const char *action_suggest(const char *word, const struct list *keywords, const char **extra);
+void free_act_rule(struct act_rule *rule);
 
 static inline struct action_kw *action_lookup(struct list *keywords, const char *kw)
 {
 	struct action_kw_list *kw_list;
-	struct action_kw *best = NULL;
-	int len, bestlen = 0;
 	int i;
 
 	if (LIST_ISEMPTY(keywords))
@@ -45,18 +47,13 @@ static inline struct action_kw *action_lookup(struct list *keywords, const char
 	list_for_each_entry(kw_list, keywords, list) {
 		for (i = 0; kw_list->kw[i].kw != NULL; i++) {
 			if ((kw_list->kw[i].flags & KWF_MATCH_PREFIX) &&
-			    (len = strlen(kw_list->kw[i].kw)) > bestlen &&
-			    strncmp(kw, kw_list->kw[i].kw, len) == 0) {
-				if (len > bestlen) {
-					bestlen = len;
-					best = &kw_list->kw[i];
-				}
-			}
+			    strncmp(kw, kw_list->kw[i].kw, strlen(kw_list->kw[i].kw)) == 0)
+				return &kw_list->kw[i];
 			if (strcmp(kw, kw_list->kw[i].kw) == 0)
 				return &kw_list->kw[i];
 		}
 	}
-	return best;
+	return NULL;
 }
 
 static inline void action_build_list(struct list *keywords,
@@ -115,6 +112,8 @@ static inline void release_timeout_action(struct act_rule *rule)
 	release_sample_expr(rule->arg.timeout.expr);
 }
 
+struct act_rule *new_act_rule(enum act_from from, const char *file, int linenum);
 void free_act_rules(struct list *rules);
+void dump_act_rules(const struct list *rules, const char *pfx);
 
 #endif /* _HAPROXY_ACTION_H */
