@@ -330,7 +330,8 @@ static inline void fd_insert(int fd, void *owner, void (*iocb)(int fd), unsigned
 	/* This must never happen and would definitely indicate a bug, in
 	 * addition to overwriting some unexpected memory areas.
 	 */
-	BUG_ON(fd < 0 || fd >= global.maxsock);
+	BUG_ON(fd < 0);
+	BUG_ON(fd >= global.maxsock);
 	BUG_ON(fdtab[fd].owner != NULL);
 	BUG_ON(fdtab[fd].state != 0);
 
