@@ -32,7 +32,6 @@
 
 #include <import/ist.h>
 #include <haproxy/chunk.h>
-#include <haproxy/global.h>
 #include <haproxy/h2.h>
 #include <haproxy/hpack-dec.h>
 #include <haproxy/hpack-huff.h>
@@ -167,7 +166,7 @@ int hpack_decode_frame(struct hpack_dht *dht, const uint8_t *raw, uint32_t len,
 		if (*raw >= 0x80) {
 			/* indexed header field */
 			if (*raw == 0x80) {
-				hpack_debug_printf("unhandled code 0x%02x (raw=%p, len=%d)\n", *raw, raw, len);
+				hpack_debug_printf("unhandled code 0x%02x (raw=%p, len=%u)\n", *raw, raw, len);
 				ret = -HPACK_ERR_UNKNOWN_OPCODE;
 				goto leave;
 			}
@@ -420,15 +419,6 @@ int hpack_decode_frame(struct hpack_dht *dht, const uint8_t *raw, uint32_t len,
 			/* <name> and <value> are correctly filled here */
 		}
 
-		/* We must not accept empty header names (forbidden by the spec and used
-		 * as a list termination).
-		 */
-		if (!name.len) {
-			hpack_debug_printf("##ERR@%d##\n", __LINE__);
-			ret = -HPACK_ERR_INVALID_ARGUMENT;
-			goto leave;
-		}
-
 		/* here's what we have here :
 		 *   - name.len > 0
 		 *   - value is filled with either const data or data allocated from tmp
