@@ -33,7 +33,8 @@
 #include <haproxy/pool.h>
 #include <haproxy/regex.h>
 #include <haproxy/sample.h>
-#include <haproxy/stream_interface.h>
+#include <haproxy/sc_strm.h>
+#include <haproxy/stconn.h>
 #include <haproxy/tools.h>
 #include <haproxy/uri_auth-t.h>
 #include <haproxy/uri_normalizer.h>
@@ -178,6 +179,7 @@ static enum act_parse_ret parse_set_req_line(const char **args, int *orig_arg, s
 	}
 	rule->action_ptr = http_action_set_req_line;
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	if (!*args[cur_arg] ||
 	    (*args[cur_arg + 1] && strcmp(args[cur_arg + 1], "if") != 0 && strcmp(args[cur_arg + 1], "unless") != 0)) {
@@ -185,7 +187,6 @@ static enum act_parse_ret parse_set_req_line(const char **args, int *orig_arg, s
 		return ACT_RET_PRS_ERR;
 	}
 
-	LIST_INIT(&rule->arg.http.fmt);
 	px->conf.args.ctx = ARGC_HRQ;
 	if (px->cap & PR_CAP_FE)
 		cap |= SMP_VAL_FE_HRQ_HDR;
@@ -538,6 +539,9 @@ static enum act_return http_action_replace_uri(struct act_rule *rule, struct pro
 		uri = http_parse_path(&parser);
 	}
 
+	if (!istlen(uri))
+		goto leave;
+
 	if (!regex_exec_match2(rule->arg.http.re, uri.ptr, uri.len, MAX_MATCH, pmatch, 0))
 		goto leave;
 
@@ -612,6 +616,7 @@ static enum act_parse_ret parse_replace_uri(const char **args, int *orig_arg, st
 
 	rule->action_ptr = http_action_replace_uri;
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	if (!*args[cur_arg] || !*args[cur_arg+1] ||
 	    (*args[cur_arg+2] && strcmp(args[cur_arg+2], "if") != 0 && strcmp(args[cur_arg+2], "unless") != 0)) {
@@ -625,7 +630,6 @@ static enum act_parse_ret parse_replace_uri(const char **args, int *orig_arg, st
 		return ACT_RET_PRS_ERR;
 	}
 
-	LIST_INIT(&rule->arg.http.fmt);
 	px->conf.args.ctx = ARGC_HRQ;
 	if (px->cap & PR_CAP_FE)
 		cap |= SMP_VAL_FE_HRQ_HDR;
@@ -676,6 +680,7 @@ static enum act_parse_ret parse_http_set_status(const char **args, int *orig_arg
 	rule->action = ACT_CUSTOM;
 	rule->action_ptr = action_http_set_status;
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	/* Check if an argument is available */
 	if (!*args[*orig_arg]) {
@@ -697,12 +702,10 @@ static enum act_parse_ret parse_http_set_status(const char **args, int *orig_arg
 	if (*args[*orig_arg] && strcmp(args[*orig_arg], "reason") == 0 &&
 	    (*args[*orig_arg + 1] && strcmp(args[*orig_arg + 1], "if") != 0 && strcmp(args[*orig_arg + 1], "unless") != 0)) {
 		(*orig_arg)++;
-		rule->arg.http.str.ptr = strdup(args[*orig_arg]);
-		rule->arg.http.str.len = strlen(rule->arg.http.str.ptr);
+		rule->arg.http.str = ist(strdup(args[*orig_arg]));
 		(*orig_arg)++;
 	}
 
-	LIST_INIT(&rule->arg.http.fmt);
 	return ACT_RET_PRS_OK;
 }
 
@@ -716,7 +719,7 @@ static enum act_parse_ret parse_http_set_status(const char **args, int *orig_arg
 static enum act_return http_action_reject(struct act_rule *rule, struct proxy *px,
                                           struct session *sess, struct stream *s, int flags)
 {
-	si_must_kill_conn(chn_prod(&s->req));
+	sc_must_kill_conn(chn_prod(&s->req));
 	channel_abort(&s->req);
 	channel_abort(&s->res);
 	s->req.analysers &= AN_REQ_FLT_END;
@@ -750,19 +753,17 @@ static enum act_parse_ret parse_http_action_reject(const char **args, int *orig_
 /* This function executes the "disable-l7-retry" HTTP action.
  * It disables L7 retries (all retry except for a connection failure). This
  * can be useful for example to avoid retrying on POST requests.
- * It just removes the L7 retry flag on the stream_interface, and always
+ * It just removes the L7 retry flag on the HTTP transaction, and always
  * return ACT_RET_CONT;
  */
 static enum act_return http_req_disable_l7_retry(struct act_rule *rule, struct proxy *px,
                                           struct session *sess, struct stream *s, int flags)
 {
-	struct stream_interface *si = &s->si[1];
-
-	/* In theory, the SI_FL_L7_RETRY flags isn't set at this point, but
+	/* In theory, the TX_L7_RETRY flags isn't set at this point, but
 	 * let's be future-proof and remove it anyway.
 	 */
-	si->flags &= ~SI_FL_L7_RETRY;
-	si->flags |= SI_FL_D_L7_RETRY;
+	s->txn->flags &= ~TX_L7_RETRY;
+	s->txn->flags |= TX_D_L7_RETRY;
 	return ACT_RET_CONT;
 }
 
@@ -1317,6 +1318,7 @@ static enum act_parse_ret parse_http_auth(const char **args, int *orig_arg, stru
 	rule->flags |= ACT_FLAG_FINAL;
 	rule->action_ptr = http_action_auth;
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	cur_arg = *orig_arg;
 	if (strcmp(args[cur_arg], "realm") == 0) {
@@ -1325,12 +1327,10 @@ static enum act_parse_ret parse_http_auth(const char **args, int *orig_arg, stru
 			memprintf(err, "missing realm value.\n");
 			return ACT_RET_PRS_ERR;
 		}
-		rule->arg.http.str.ptr = strdup(args[cur_arg]);
-		rule->arg.http.str.len = strlen(rule->arg.http.str.ptr);
+		rule->arg.http.str = ist(strdup(args[cur_arg]));
 		cur_arg++;
 	}
 
-	LIST_INIT(&rule->arg.http.fmt);
 	*orig_arg = cur_arg;
 	return ACT_RET_PRS_OK;
 }
@@ -1348,7 +1348,7 @@ static enum act_parse_ret parse_http_auth(const char **args, int *orig_arg, stru
 static enum act_return http_action_early_hint(struct act_rule *rule, struct proxy *px,
 					      struct session *sess, struct stream *s, int flags)
 {
-	struct act_rule *prev_rule, *next_rule;
+	struct act_rule *next_rule;
 	struct channel *res = &s->res;
 	struct htx *htx = htx_from_buf(&res->buf);
 	struct buffer *value = alloc_trash_chunk();
@@ -1363,13 +1363,10 @@ static enum act_return http_action_early_hint(struct act_rule *rule, struct prox
 		goto error;
 	}
 
-	/* get previous and next rules */
-	prev_rule = LIST_PREV(&rule->list, typeof(rule), list);
-	next_rule = LIST_NEXT(&rule->list, typeof(rule), list);
-
-	/* if no previous rule or previous rule is not early-hint, start a new response. Otherwise,
-	 * continue to add link to a previously started response */
-	if (&prev_rule->list == s->current_rule_list || prev_rule->action_ptr != http_action_early_hint) {
+	/* if there is no pending 103 response, start a new response. Otherwise,
+	 * continue to add link to a previously started response
+         */
+	if (s->txn->status != 103) {
 		struct htx_sl *sl;
 		unsigned int flags = (HTX_SL_F_IS_RESP|HTX_SL_F_VER_11|
 				      HTX_SL_F_XFER_LEN|HTX_SL_F_BODYLESS);
@@ -1379,6 +1376,7 @@ static enum act_return http_action_early_hint(struct act_rule *rule, struct prox
 		if (!sl)
 			goto error;
 		sl->info.res.status = 103;
+		s->txn->status = 103;
 	}
 
 	/* Add the HTTP Early Hint HTTP 103 response heade */
@@ -1386,18 +1384,16 @@ static enum act_return http_action_early_hint(struct act_rule *rule, struct prox
 	if (!htx_add_header(htx, rule->arg.http.str, ist2(b_head(value), b_data(value))))
 		goto error;
 
-	/* if it is the last rule or the next one is not an early-hint, terminate the current
-	 * response. */
-	if (&next_rule->list == s->current_rule_list || next_rule->action_ptr != http_action_early_hint) {
-		if (!htx_add_endof(htx, HTX_BLK_EOH)) {
-			/* If an error occurred during an Early-hint rule,
-			 * remove the incomplete HTTP 103 response from the
-			 * buffer */
+	/* if it is the last rule or the next one is not an early-hint or an
+	 * conditional early-hint, terminate the current response.
+	 */
+	next_rule = LIST_NEXT(&rule->list, typeof(rule), list);
+	if (&next_rule->list == s->current_rule_list || next_rule->action_ptr != http_action_early_hint || next_rule->cond) {
+		if (!htx_add_endof(htx, HTX_BLK_EOH))
 			goto error;
-		}
-
 		if (!http_forward_proxy_resp(s, 0))
 			goto error;
+		s->txn->status = 0;
 	}
 
   leave:
@@ -1409,6 +1405,7 @@ static enum act_return http_action_early_hint(struct act_rule *rule, struct prox
 	 * HTTP 103 response from the buffer */
 	channel_htx_truncate(res, htx);
 	ret = ACT_RET_ERR;
+	s->txn->status = 0;
 	goto leave;
 }
 
@@ -1500,6 +1497,7 @@ static enum act_parse_ret parse_http_set_header(const char **args, int *orig_arg
 		rule->action_ptr = http_action_set_header;
 	}
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	cur_arg = *orig_arg;
 	if (!*args[cur_arg] || !*args[cur_arg+1]) {
@@ -1508,9 +1506,7 @@ static enum act_parse_ret parse_http_set_header(const char **args, int *orig_arg
 	}
 
 
-	rule->arg.http.str.ptr = strdup(args[cur_arg]);
-	rule->arg.http.str.len = strlen(rule->arg.http.str.ptr);
-	LIST_INIT(&rule->arg.http.fmt);
+	rule->arg.http.str = ist(strdup(args[cur_arg]));
 
 	if (rule->from == ACT_F_HTTP_REQ) {
 		px->conf.args.ctx = ARGC_HRQ;
@@ -1610,6 +1606,7 @@ static enum act_parse_ret parse_http_replace_header(const char **args, int *orig
 		rule->action = 1; // replace-value
 	rule->action_ptr = http_action_replace_header;
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	cur_arg = *orig_arg;
 	if (!*args[cur_arg] || !*args[cur_arg+1] || !*args[cur_arg+2]) {
@@ -1617,9 +1614,7 @@ static enum act_parse_ret parse_http_replace_header(const char **args, int *orig
 		return ACT_RET_PRS_ERR;
 	}
 
-	rule->arg.http.str.ptr = strdup(args[cur_arg]);
-	rule->arg.http.str.len = strlen(rule->arg.http.str.ptr);
-	LIST_INIT(&rule->arg.http.fmt);
+	rule->arg.http.str = ist(strdup(args[cur_arg]));
 
 	cur_arg++;
 	if (!(rule->arg.http.re = regex_comp(args[cur_arg], 1, 1, err))) {
@@ -1714,6 +1709,7 @@ static enum act_parse_ret parse_http_del_header(const char **args, int *orig_arg
 	rule->action = PAT_MATCH_STR;
 	rule->action_ptr = http_action_del_header;
 	rule->release_ptr = release_http_action;
+	LIST_INIT(&rule->arg.http.fmt);
 
 	cur_arg = *orig_arg;
 	if (!*args[cur_arg]) {
@@ -1721,11 +1717,9 @@ static enum act_parse_ret parse_http_del_header(const char **args, int *orig_arg
 		return ACT_RET_PRS_ERR;
 	}
 
-	rule->arg.http.str.ptr = strdup(args[cur_arg]);
-	rule->arg.http.str.len = strlen(rule->arg.http.str.ptr);
+	rule->arg.http.str = ist(strdup(args[cur_arg]));
 	px->conf.args.ctx = (rule->from == ACT_F_HTTP_REQ ? ARGC_HRQ : ARGC_HRS);
 
-	LIST_INIT(&rule->arg.http.fmt);
 	if (strcmp(args[cur_arg+1], "-m") == 0) {
 		cur_arg++;
 		if (!*args[cur_arg+1]) {
@@ -1763,6 +1757,9 @@ static void release_http_redir(struct act_rule *rule)
 	struct redirect_rule *redir;
 
 	redir = rule->arg.redir;
+	if (!redir)
+		return;
+
 	LIST_DELETE(&redir->list);
 	if (redir->cond) {
 		prune_acl_cond(redir->cond);
