@@ -52,7 +52,7 @@ struct quic_dghdlr *quic_dghdlrs;
 /* Size of the internal buffer of QUIC RX buffer at the fd level */
 #define QUIC_RX_BUFSZ  (1UL << 18)
 
-DECLARE_STATIC_POOL(pool_head_quic_rxbuf, "quic_rxbuf_pool", QUIC_RX_BUFSZ);
+DECLARE_STATIC_POOL(pool_head_quic_rxbuf, "quic_rxbuf", QUIC_RX_BUFSZ);
 
 static void quic_add_listener(struct protocol *proto, struct listener *listener);
 static int quic_bind_listener(struct listener *listener, char *errmsg, int errlen);
@@ -703,7 +703,7 @@ static int quic_alloc_dghdlrs(void)
 {
 	int i;
 
-	quic_dghdlrs = calloc(global.nbthread, sizeof(struct quic_dghdlr));
+	quic_dghdlrs = calloc(global.nbthread, sizeof(*quic_dghdlrs));
 	if (!quic_dghdlrs) {
 		ha_alert("Failed to allocate the quic datagram handlers.\n");
 		return 0;
