@@ -27,7 +27,7 @@
 #include <haproxy/buf.h>
 #include <haproxy/chunk.h>
 #include <haproxy/h3.h>
-#include <haproxy/ncbuf.h>
+#include <haproxy/mux_quic.h>
 #include <haproxy/qpack-t.h>
 #include <haproxy/qpack-dec.h>
 #include <haproxy/qpack-tbl.h>
@@ -97,22 +97,33 @@ static uint64_t qpack_get_varint(const unsigned char **buf, uint64_t *len_in, in
  *
  * Returns 0 on success else non-zero.
  */
-int qpack_decode_enc(struct qcs *qcs, void *ctx)
+int qpack_decode_enc(struct buffer *buf, int fin, void *ctx)
 {
+	struct qcs *qcs = ctx;
 	size_t len;
-	struct ncbuf *rxbuf;
 	unsigned char inst;
 
-	rxbuf = &qcs->rx.ncbuf;
-	len = ncb_data(rxbuf, 0);
-	qpack_debug_hexdump(stderr, "[QPACK-DEC-ENC] ", ncb_head(rxbuf), 0, len);
+	/* RFC 9204 4.2. Encoder and Decoder Streams
+	 *
+	 * The sender MUST NOT close either of these streams, and the receiver
+	 * MUST NOT request that the sender close either of these streams.
+	 * Closure of either unidirectional stream type MUST be treated as a
+	 * connection error of type H3_CLOSED_CRITICAL_STREAM.
+	 */
+	if (fin) {
+		qcc_emit_cc_app(qcs->qcc, H3_CLOSED_CRITICAL_STREAM, 1);
+		return -1;
+	}
+
+	len = b_data(buf);
+	qpack_debug_hexdump(stderr, "[QPACK-DEC-ENC] ", b_head(buf), 0, len);
 
 	if (!len) {
 		qpack_debug_printf(stderr, "[QPACK-DEC-ENC] empty stream\n");
 		return 0;
 	}
 
-	inst = (unsigned char)*ncb_head(rxbuf) & QPACK_ENC_INST_BITMASK;
+	inst = (unsigned char)*b_head(buf) & QPACK_ENC_INST_BITMASK;
 	if (inst == QPACK_ENC_INST_DUP) {
 		/* Duplicate */
 	}
@@ -133,22 +144,33 @@ int qpack_decode_enc(struct qcs *qcs, void *ctx)
  *
  * Returns 0 on success else non-zero.
  */
-int qpack_decode_dec(struct qcs *qcs, void *ctx)
+int qpack_decode_dec(struct buffer *buf, int fin, void *ctx)
 {
+	struct qcs *qcs = ctx;
 	size_t len;
-	struct ncbuf *rxbuf;
 	unsigned char inst;
 
-	rxbuf = &qcs->rx.ncbuf;
-	len = ncb_data(rxbuf, 0);
-	qpack_debug_hexdump(stderr, "[QPACK-DEC-DEC] ", ncb_head(rxbuf), 0, len);
+	/* RFC 9204 4.2. Encoder and Decoder Streams
+	 *
+	 * The sender MUST NOT close either of these streams, and the receiver
+	 * MUST NOT request that the sender close either of these streams.
+	 * Closure of either unidirectional stream type MUST be treated as a
+	 * connection error of type H3_CLOSED_CRITICAL_STREAM.
+	 */
+	if (fin) {
+		qcc_emit_cc_app(qcs->qcc, H3_CLOSED_CRITICAL_STREAM, 1);
+		return -1;
+	}
+
+	len = b_data(buf);
+	qpack_debug_hexdump(stderr, "[QPACK-DEC-DEC] ", b_head(buf), 0, len);
 
 	if (!len) {
 		qpack_debug_printf(stderr, "[QPACK-DEC-DEC] empty stream\n");
 		return 0;
 	}
 
-	inst = (unsigned char)*ncb_head(rxbuf) & QPACK_DEC_INST_BITMASK;
+	inst = (unsigned char)*b_head(buf) & QPACK_DEC_INST_BITMASK;
 	if (inst == QPACK_DEC_INST_ICINC) {
 		/* Insert count increment */
 	}
@@ -186,12 +208,13 @@ static int qpack_decode_fs_pfx(uint64_t *enc_ric, uint64_t *db, int *sign_bit,
  * a storage for some elements pointing into it. An end marker is inserted at
  * the end of the list with empty strings as name/value.
  *
- * Returns 0 on success. In case of error, a negative code QPACK_ERR_* is
- * returned.
+ * Returns the number of headers inserted into list excluding the end marker.
+ * In case of error, a negative code QPACK_ERR_* is returned.
  */
 int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
                     struct http_hdr *list, int list_size)
 {
+	struct ist name, value;
 	uint64_t enc_ric, db;
 	int s;
 	unsigned int efl_type;
@@ -256,7 +279,6 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				goto out;
 			}
 
-			/* XXX Value string XXX */
 			raw += length;
 			len -= length;
 #endif
@@ -302,10 +324,10 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 		else if (efl_type & QPACK_IFL_BIT) {
 			/* Indexed field line */
 			uint64_t index;
-			unsigned int t;
+			unsigned int static_tbl;
 
 			qpack_debug_printf(stderr, "indexed field line:");
-			t = efl_type & 0x40;
+			static_tbl = efl_type & 0x40;
 			index = qpack_get_varint(&raw, &len, 6);
 			if (len == (uint64_t)-1) {
 				qpack_debug_printf(stderr, "##ERR@%d\n", __LINE__);
@@ -313,8 +335,10 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				goto out;
 			}
 
-			if (t)
-				list[hdr_idx++] = qpack_sht[index];
+			if (static_tbl) {
+				name = qpack_sht[index].n;
+				value = qpack_sht[index].v;
+			}
 			else {
 				/* RFC9204 2.2.3 Invalid References
 				 *
@@ -329,16 +353,16 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				return -QPACK_DECOMPRESSION_FAILED;
 			}
 
-			qpack_debug_printf(stderr,  " t=%d index=%llu", !!t, (unsigned long long)index);
+			qpack_debug_printf(stderr,  " t=%d index=%llu", !!static_tbl, (unsigned long long)index);
 		}
 		else if (efl_type & QPACK_LFL_WNR_BIT) {
 			/* Literal field line with name reference */
 			uint64_t index, length;
-			unsigned int t, n __maybe_unused, h;
+			unsigned int static_tbl, n __maybe_unused, h;
 
 			qpack_debug_printf(stderr, "Literal field line with name reference:");
 			n = efl_type & 0x20;
-			t = efl_type & 0x10;
+			static_tbl = efl_type & 0x10;
 			index = qpack_get_varint(&raw, &len, 4);
 			if (len == (uint64_t)-1) {
 				qpack_debug_printf(stderr, "##ERR@%d\n", __LINE__);
@@ -346,8 +370,9 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				goto out;
 			}
 
-			if (t)
-				list[hdr_idx] = qpack_sht[index];
+			if (static_tbl) {
+				name = qpack_sht[index].n;
+			}
 			else {
 				/* RFC9204 2.2.3 Invalid References
 				 *
@@ -362,7 +387,7 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				return -QPACK_DECOMPRESSION_FAILED;
 			}
 
-			qpack_debug_printf(stderr, " n=%d t=%d index=%llu", !!n, !!t, (unsigned long long)index);
+			qpack_debug_printf(stderr, " n=%d t=%d index=%llu", !!n, !!static_tbl, (unsigned long long)index);
 			h = *raw & 0x80;
 			length = qpack_get_varint(&raw, &len, 7);
 			if (len == (uint64_t)-1) {
@@ -392,10 +417,10 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				qpack_debug_printf(stderr, " [name huff %d->%d '%s']", (int)length, (int)nlen, trash);
 				/* makes an ist from tmp storage */
 				b_add(tmp, nlen);
-				list[hdr_idx].v = ist2(trash, nlen);
+				value = ist2(trash, nlen);
 			}
 			else {
-				list[hdr_idx].v = ist2(raw, length);
+				value = ist2(raw, length);
 			}
 
 			if (len < length) {
@@ -406,7 +431,6 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 
 			raw += length;
 			len -= length;
-			++hdr_idx;
 		}
 		else if (efl_type & QPACK_LFL_WLN_BIT) {
 			/* Literal field line with literal name */
@@ -452,14 +476,15 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				qpack_debug_printf(stderr, " [name huff %d->%d '%s']", (int)name_len, (int)nlen, trash);
 				/* makes an ist from tmp storage */
 				b_add(tmp, nlen);
-				list[hdr_idx].n = ist2(trash, nlen);
+				name = ist2(trash, nlen);
 			}
 			else {
-				list[hdr_idx].n = ist2(raw, name_len);
+				name = ist2(raw, name_len);
 			}
 
 			raw += name_len;
 			len -= name_len;
+
 			hvalue = *raw & 0x80;
 			value_len = qpack_get_varint(&raw, &len, 7);
 			if (len == (uint64_t)-1) {
@@ -496,17 +521,20 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 				qpack_debug_printf(stderr, " [name huff %d->%d '%s']", (int)value_len, (int)nlen, trash);
 				/* makes an ist from tmp storage */
 				b_add(tmp, nlen);
-				list[hdr_idx].v = ist2(trash, nlen);
+				value = ist2(trash, nlen);
 			}
 			else {
-				list[hdr_idx].v = ist2(raw, value_len);
+				value = ist2(raw, value_len);
 			}
 
 			raw += value_len;
 			len -= value_len;
-
-			++hdr_idx;
 		}
+
+		list[hdr_idx].n = name;
+		list[hdr_idx].v = value;
+		++hdr_idx;
+
 		qpack_debug_printf(stderr, "\n");
 	}
 
@@ -518,6 +546,7 @@ int qpack_decode_fs(const unsigned char *raw, uint64_t len, struct buffer *tmp,
 
 	/* put an end marker */
 	list[hdr_idx].n = list[hdr_idx].v = IST_NULL;
+	ret = hdr_idx;
 
  out:
 	qpack_debug_printf(stderr, "-- done: ret=%d\n", ret);
