@@ -3,7 +3,7 @@
  *
  * This file contains definitions for QUIC congestion control.
  *
- * Copyright 2019 HAProxy Technologies, Frédéric Lécaille <flecaille@haproxy.com>
+ * Copyright 2019 HAProxy Technologies, Frederic Lecaille <flecaille@haproxy.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,93 +26,105 @@
 
 #define TRACE_SOURCE    &trace_quic
 
+/* Newreno state */
+struct nr {
+	uint32_t ssthresh;
+	uint32_t recovery_start_time;
+	uint32_t remain_acked;
+};
+
 static int quic_cc_nr_init(struct quic_cc *cc)
 {
-	struct quic_path *path;
+	struct nr *nr = quic_cc_priv(cc);
 
-	path = container_of(cc, struct quic_path, cc);
-	cc->algo_state.nr.state = QUIC_CC_ST_SS;
-	cc->algo_state.nr.cwnd = path->cwnd;
-	cc->algo_state.nr.ssthresh = QUIC_CC_INFINITE_SSTHESH;
-	cc->algo_state.nr.recovery_start_time = 0;
+	cc->algo->state = QUIC_CC_ST_SS;
+	nr->ssthresh = QUIC_CC_INFINITE_SSTHESH;
+	nr->recovery_start_time = 0;
+	nr->remain_acked = 0;
 
 	return 1;
 }
 
+/* Re-enter slow start state. */
+static void quic_cc_nr_slow_start(struct quic_cc *cc)
+{
+	struct quic_path *path;
+	struct nr *nr = quic_cc_priv(cc);
+
+	path = container_of(cc, struct quic_path, cc);
+	path->cwnd = path->min_cwnd;
+	/* Re-entering slow start state. */
+	cc->algo->state = QUIC_CC_ST_SS;
+	/* Recovery start time reset */
+	nr->recovery_start_time = 0;
+}
+
 /* Slow start callback. */
 static void quic_cc_nr_ss_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 {
 	struct quic_path *path;
+	struct nr *nr = quic_cc_priv(cc);
 
-	TRACE_ENTER(QUIC_EV_CONN_CC, cc->qc->conn, ev);
+	TRACE_ENTER(QUIC_EV_CONN_CC, cc->qc, ev);
 	path = container_of(cc, struct quic_path, cc);
 	switch (ev->type) {
 	case QUIC_CC_EVT_ACK:
-		path->in_flight -= ev->ack.acked;
 		/* Do not increase the congestion window in recovery period. */
-		if (ev->ack.time_sent <= cc->algo_state.nr.recovery_start_time)
+		if (ev->ack.time_sent <= nr->recovery_start_time)
 			return;
 
-		cc->algo_state.nr.cwnd += ev->ack.acked;
+		path->cwnd += ev->ack.acked;
 		/* Exit to congestion avoidance if slow start threshold is reached. */
-		if (cc->algo_state.nr.cwnd > cc->algo_state.nr.ssthresh)
-			cc->algo_state.nr.state = QUIC_CC_ST_CA;
-		path->cwnd = cc->algo_state.nr.cwnd;
+		if (path->cwnd > nr->ssthresh)
+			cc->algo->state = QUIC_CC_ST_CA;
 		break;
 
 	case QUIC_CC_EVT_LOSS:
-		path->in_flight -= ev->loss.lost_bytes;
-		cc->algo_state.nr.cwnd = QUIC_MAX(cc->algo_state.nr.cwnd >> 1, path->min_cwnd);
-		path->cwnd = cc->algo_state.nr.ssthresh = cc->algo_state.nr.cwnd;
+		path->cwnd = QUIC_MAX(path->cwnd >> 1, path->min_cwnd);
+		nr->ssthresh = path->cwnd;
 		/* Exit to congestion avoidance. */
-		cc->algo_state.nr.state = QUIC_CC_ST_CA;
+		cc->algo->state = QUIC_CC_ST_CA;
 		break;
 
 	case QUIC_CC_EVT_ECN_CE:
 		/* XXX TO DO XXX */
 		break;
 	}
-	TRACE_LEAVE(QUIC_EV_CONN_CC, cc->qc->conn,, cc);
+	TRACE_LEAVE(QUIC_EV_CONN_CC, cc->qc,, cc);
 }
 
 /* Congestion avoidance callback. */
 static void quic_cc_nr_ca_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 {
 	struct quic_path *path;
+	struct nr *nr = quic_cc_priv(cc);
 
-	TRACE_ENTER(QUIC_EV_CONN_CC, cc->qc->conn, ev);
+	TRACE_ENTER(QUIC_EV_CONN_CC, cc->qc, ev);
 	path = container_of(cc, struct quic_path, cc);
 	switch (ev->type) {
 	case QUIC_CC_EVT_ACK:
-		path->in_flight -= ev->ack.acked;
+	{
+		uint64_t acked;
 		/* Do not increase the congestion window in recovery period. */
-		if (ev->ack.time_sent <= cc->algo_state.nr.recovery_start_time)
+		if (ev->ack.time_sent <= nr->recovery_start_time)
 			goto out;
 
-		/* Increasing the congestion window by 1 maximum packet size by
-		 * congestion window.
+		/* Increasing the congestion window by (acked / cwnd)
 		 */
-		cc->algo_state.nr.cwnd +=
-			path->mtu * QUIC_MAX(1ULL, (unsigned long long)ev->ack.acked / cc->algo_state.nr.cwnd);
-		path->cwnd = cc->algo_state.nr.cwnd;
+		acked = ev->ack.acked * path->mtu + nr->remain_acked;
+		nr->remain_acked = acked % path->cwnd;
+		path->cwnd += acked / path->cwnd;
 		break;
+	}
 
 	case QUIC_CC_EVT_LOSS:
-		path->in_flight -= ev->loss.lost_bytes;
-		if (ev->loss.newest_time_sent > cc->algo_state.nr.recovery_start_time) {
-			cc->algo_state.nr.recovery_start_time = ev->loss.now_ms;
-			cc->algo_state.nr.cwnd = QUIC_MAX(cc->algo_state.nr.cwnd >> 1, path->min_cwnd);
-			cc->algo_state.nr.ssthresh = cc->algo_state.nr.cwnd;
-		}
-		if (quic_loss_persistent_congestion(&path->loss,
-		                                    ev->loss.period,
-		                                    ev->loss.now_ms,
-		                                    ev->loss.max_ack_delay)) {
-			cc->algo_state.nr.cwnd = path->min_cwnd;
-			/* Re-entering slow start state. */
-			cc->algo_state.nr.state = QUIC_CC_ST_SS;
-		}
-		path->cwnd = cc->algo_state.nr.cwnd;
+		/* Do not decrease the congestion window when already in recovery period. */
+		if (ev->loss.time_sent <= nr->recovery_start_time)
+			goto out;
+
+		nr->recovery_start_time = now_ms;
+		nr->ssthresh = path->cwnd;
+		path->cwnd = QUIC_MAX(path->cwnd >> 1, path->min_cwnd);
 		break;
 
 	case QUIC_CC_EVT_ECN_CE:
@@ -121,16 +133,20 @@ static void quic_cc_nr_ca_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 	}
 
  out:
-	TRACE_LEAVE(QUIC_EV_CONN_CC, cc->qc->conn, NULL, cc);
+	TRACE_LEAVE(QUIC_EV_CONN_CC, cc->qc, NULL, cc);
 }
 
 static void quic_cc_nr_state_trace(struct buffer *buf, const struct quic_cc *cc)
 {
+	struct quic_path *path;
+	struct nr *nr = quic_cc_priv(cc);
+
+	path = container_of(cc, struct quic_path, cc);
 	chunk_appendf(buf, " state=%s cwnd=%llu ssthresh=%ld recovery_start_time=%llu",
-	              quic_cc_state_str(cc->algo_state.nr.state),
-	              (unsigned long long)cc->algo_state.nr.cwnd,
-	              (long)cc->algo_state.nr.ssthresh,
-	              (unsigned long long)cc->algo_state.nr.recovery_start_time);
+	              quic_cc_state_str(cc->algo->state),
+	              (unsigned long long)path->cwnd,
+	              (long)nr->ssthresh,
+	              (unsigned long long)nr->recovery_start_time);
 }
 
 static void (*quic_cc_nr_state_cbs[])(struct quic_cc *cc,
@@ -141,13 +157,14 @@ static void (*quic_cc_nr_state_cbs[])(struct quic_cc *cc,
 
 static void quic_cc_nr_event(struct quic_cc *cc, struct quic_cc_event *ev)
 {
-	return quic_cc_nr_state_cbs[cc->algo_state.nr.state](cc, ev);
+	return quic_cc_nr_state_cbs[cc->algo->state](cc, ev);
 }
 
 struct quic_cc_algo quic_cc_algo_nr = {
 	.type        = QUIC_CC_ALGO_TP_NEWRENO,
 	.init        = quic_cc_nr_init,
 	.event       = quic_cc_nr_event,
+	.slow_start  = quic_cc_nr_slow_start,
 	.state_trace = quic_cc_nr_state_trace,
 };
 
