@@ -189,7 +189,7 @@ void chunk_frm_appendf(struct buffer *buf, const struct quic_frame *frm)
 	case QUIC_FT_CONNECTION_CLOSE:
 	{
 		const struct quic_connection_close *cc = &frm->connection_close;
-		size_t plen = QUIC_MIN(cc->reason_phrase_len, sizeof cc->reason_phrase);
+		size_t plen = QUIC_MIN((size_t)cc->reason_phrase_len, sizeof cc->reason_phrase);
 		chunk_appendf(&trace_buf,
 		              " error_code=%llu frame_type=%llu reason_phrase_len=%llu",
 		              (ull)cc->error_code, (ull)cc->frame_type,
@@ -201,7 +201,7 @@ void chunk_frm_appendf(struct buffer *buf, const struct quic_frame *frm)
 	case QUIC_FT_CONNECTION_CLOSE_APP:
 	{
 		const struct quic_connection_close_app *cc = &frm->connection_close_app;
-		size_t plen = QUIC_MIN(cc->reason_phrase_len, sizeof cc->reason_phrase);
+		size_t plen = QUIC_MIN((size_t)cc->reason_phrase_len, sizeof cc->reason_phrase);
 		chunk_appendf(&trace_buf,
 		              " error_code=%llu reason_phrase_len=%llu",
 		              (ull)cc->error_code, (ull)cc->reason_phrase_len);
@@ -279,7 +279,7 @@ static int quic_build_ack_frame(unsigned char **buf, const unsigned char *end,
 
 	ar = eb64_last(&tx_ack->arngs->root);
 	ar_node = eb64_entry(ar, struct quic_arng_node, first);
-	TRACE_PROTO("ack range", QUIC_EV_CONN_PRSAFRM,
+	TRACE_DEVEL("ack range", QUIC_EV_CONN_PRSAFRM,
 	            qc,, &ar_node->last, &ar_node->first.key);
 	if (!quic_enc_int(buf, end, ar_node->last) ||
 	    !quic_enc_int(buf, end, tx_ack->ack_delay) ||
@@ -289,7 +289,7 @@ static int quic_build_ack_frame(unsigned char **buf, const unsigned char *end,
 
 	while ((prev_ar = eb64_prev(ar))) {
 		prev_ar_node = eb64_entry(prev_ar, struct quic_arng_node, first);
-		TRACE_PROTO("ack range", QUIC_EV_CONN_PRSAFRM, qc,,
+		TRACE_DEVEL("ack range", QUIC_EV_CONN_PRSAFRM, qc,,
 		            &prev_ar_node->last, &prev_ar_node->first.key);
 		if (!quic_enc_int(buf, end, ar_node->first.key - prev_ar_node->last - 2) ||
 		    !quic_enc_int(buf, end, prev_ar_node->last - prev_ar_node->first.key))
@@ -925,7 +925,7 @@ static int quic_parse_connection_close_frame(struct quic_frame *frm, struct quic
 	    end - *buf < cc->reason_phrase_len)
 		return 0;
 
-	plen = QUIC_MIN(cc->reason_phrase_len, sizeof cc->reason_phrase);
+	plen = QUIC_MIN((size_t)cc->reason_phrase_len, sizeof cc->reason_phrase);
 	memcpy(cc->reason_phrase, *buf, plen);
 	*buf += cc->reason_phrase_len;
 
@@ -969,7 +969,7 @@ static int quic_parse_connection_close_app_frame(struct quic_frame *frm, struct
 	    end - *buf < cc->reason_phrase_len)
 		return 0;
 
-	plen = QUIC_MIN(cc->reason_phrase_len, sizeof cc->reason_phrase);
+	plen = QUIC_MIN((size_t)cc->reason_phrase_len, sizeof cc->reason_phrase);
 	memcpy(cc->reason_phrase, *buf, plen);
 	*buf += cc->reason_phrase_len;
 
@@ -1085,66 +1085,80 @@ int qc_parse_frm(struct quic_frame *frm, struct quic_rx_packet *pkt,
                  const unsigned char **buf, const unsigned char *end,
                  struct quic_conn *qc)
 {
+	int ret = 0;
 	const struct quic_frame_parser *parser;
 
+	TRACE_ENTER(QUIC_EV_CONN_PRSFRM, qc);
 	if (end <= *buf) {
 		TRACE_DEVEL("wrong frame", QUIC_EV_CONN_PRSFRM, qc);
-		return 0;
+		goto leave;
 	}
 
 	frm->type = *(*buf)++;
 	if (frm->type >= QUIC_FT_MAX) {
 		TRACE_DEVEL("wrong frame type", QUIC_EV_CONN_PRSFRM, qc, frm);
-		return 0;
+		goto leave;
 	}
 
 	parser = &quic_frame_parsers[frm->type];
 	if (!(parser->mask & (1U << pkt->type))) {
 		TRACE_DEVEL("unauthorized frame", QUIC_EV_CONN_PRSFRM, qc, frm);
-		return 0;
+		goto leave;
 	}
 
 	TRACE_PROTO("frame", QUIC_EV_CONN_PRSFRM, qc, frm);
 	if (!parser->func(frm, qc, buf, end)) {
 		TRACE_DEVEL("parsing error", QUIC_EV_CONN_PRSFRM, qc, frm);
-		return 0;
+		goto leave;
 	}
 
 	pkt->flags |= parser->flags;
 
-	return 1;
+	ret = 1;
+ leave:
+	TRACE_LEAVE(QUIC_EV_CONN_PRSFRM, qc);
+	return ret;
 }
 
 /* Encode <frm> QUIC frame into <buf> buffer.
  * Returns 1 if succeeded (enough room in <buf> to encode the frame), 0 if not.
+ * The buffer is updated to point to one byte past the end of the built frame
+ * only if succeeded.
  */
 int qc_build_frm(unsigned char **buf, const unsigned char *end,
                  struct quic_frame *frm, struct quic_tx_packet *pkt,
                  struct quic_conn *qc)
 {
+	int ret = 0;
 	const struct quic_frame_builder *builder;
+	unsigned char *pos = *buf;
 
+	TRACE_ENTER(QUIC_EV_CONN_BFRM, qc);
 	builder = &quic_frame_builders[frm->type];
 	if (!(builder->mask & (1U << pkt->type))) {
 		/* XXX This it a bug to send an unauthorized frame with such a packet type XXX */
-		TRACE_DEVEL("frame skipped", QUIC_EV_CONN_BFRM, qc, frm);
+		TRACE_ERROR("unauthorized frame", QUIC_EV_CONN_BFRM, qc, frm);
 		BUG_ON(!(builder->mask & (1U << pkt->type)));
 	}
 
-	if (end <= *buf) {
+	if (end <= pos) {
 		TRACE_DEVEL("not enough room", QUIC_EV_CONN_BFRM, qc, frm);
-		return 0;
+		goto leave;
 	}
 
 	TRACE_PROTO("frame", QUIC_EV_CONN_BFRM, qc, frm);
-	*(*buf)++ = frm->type;
-	if (!quic_frame_builders[frm->type].func(buf, end, frm, qc)) {
+	*pos++ = frm->type;
+	if (!quic_frame_builders[frm->type].func(&pos, end, frm, qc)) {
 		TRACE_DEVEL("frame building error", QUIC_EV_CONN_BFRM, qc, frm);
-		return 0;
+		goto leave;
 	}
 
 	pkt->flags |= builder->flags;
+	*buf = pos;
 
-	return 1;
+	ret = 1;
+ leave:
+	TRACE_LEAVE(QUIC_EV_CONN_BFRM, qc);
+	return ret;
 }
 
