@@ -318,7 +318,7 @@ void quic_sock_fd_iocb(int fd)
 		struct quic_dgram *dgram;
 
 		/* Do no mark <buf> as full, and do not try to consume it
-		 * if the contiguous remmaining space is not at the end
+		 * if the contiguous remaining space is not at the end
 		 */
 		if (b_tail(buf) + cspace < b_wrap(buf))
 			goto out;
@@ -326,11 +326,16 @@ void quic_sock_fd_iocb(int fd)
 		/* Allocate a fake datagram, without data to locate
 		 * the end of the RX buffer (required during purging).
 		 */
-		dgram = pool_zalloc(pool_head_quic_dgram);
+		dgram = pool_alloc(pool_head_quic_dgram);
 		if (!dgram)
 			goto out;
 
+		/* Initialize only the useful members of this fake datagram. */
+		dgram->buf = NULL;
 		dgram->len = cspace;
+		/* Append this datagram only to the RX buffer list. It will
+		 * not be treated by any datagram handler.
+		 */
 		LIST_APPEND(&rxbuf->dgrams, &dgram->list);
 
 		/* Consume the remaining space */
@@ -366,13 +371,14 @@ void quic_sock_fd_iocb(int fd)
 
 /* Send a datagram stored into <buf> buffer with <sz> as size.
  * The caller must ensure there is at least <sz> bytes in this buffer.
- * Return the size of this datagram if succeeded, 0 if truncated and -1 in case of
- * any error.
+ *
+ * Returns 0 on success else non-zero.
+ *
  * TODO standardize this function for a generic UDP sendto wrapper. This can be
  * done by removing the <qc> arg and replace it with address/port.
  */
-size_t qc_snd_buf(struct quic_conn *qc, const struct buffer *buf, size_t sz,
-                  int flags)
+int qc_snd_buf(struct quic_conn *qc, const struct buffer *buf, size_t sz,
+               int flags)
 {
 	ssize_t ret;
 
@@ -382,28 +388,36 @@ size_t qc_snd_buf(struct quic_conn *qc, const struct buffer *buf, size_t sz,
 		             (struct sockaddr *)&qc->peer_addr, get_addr_len(&qc->peer_addr));
 	} while (ret < 0 && errno == EINTR);
 
-	if (ret > 0) {
-		if (ret != sz)
-			return 0;
+	if (ret < 0 || ret != sz) {
+		/* TODO adjust errno for UDP context. */
+		if (errno == EAGAIN || errno == EWOULDBLOCK ||
+		    errno == ENOTCONN || errno == EINPROGRESS || errno == EBADF) {
+			struct proxy *prx = qc->li->bind_conf->frontend;
+			struct quic_counters *prx_counters =
+			  EXTRA_COUNTERS_GET(prx->extra_counters_fe,
+			                     &quic_stats_module);
 
-		/* we count the total bytes sent, and the send rate for 32-byte
-		 * blocks. The reason for the latter is that freq_ctr are
-		 * limited to 4GB and that it's not enough per second.
-		 */
-		_HA_ATOMIC_ADD(&global.out_bytes, ret);
-		update_freq_ctr(&global.out_32bps, (ret + 16) / 32);
-	}
-	else if (ret == 0 || errno == EAGAIN || errno == EWOULDBLOCK ||
-	         errno == ENOTCONN || errno == EINPROGRESS || errno == EBADF) {
-		/* TODO must be handle properly. It is justified for UDP ? */
-		qc->sendto_err++;
-	}
-	else if (errno) {
-		/* TODO unlisted errno : handle it explicitely. */
-		ABORT_NOW();
+			if (errno == EAGAIN || errno == EWOULDBLOCK)
+				HA_ATOMIC_INC(&prx_counters->socket_full);
+			else
+				HA_ATOMIC_INC(&prx_counters->sendto_err);
+		}
+		else if (errno) {
+			/* TODO unlisted errno : handle it explicitely. */
+			ABORT_NOW();
+		}
+
+		return 1;
 	}
 
-	return ret;
+	/* we count the total bytes sent, and the send rate for 32-byte blocks.
+	 * The reason for the latter is that freq_ctr are limited to 4GB and
+	 * that it's not enough per second.
+	 */
+	_HA_ATOMIC_ADD(&global.out_bytes, ret);
+	update_freq_ctr(&global.out_32bps, (ret + 16) / 32);
+
+	return 0;
 }
 
 
@@ -465,7 +479,8 @@ static int quic_alloc_accept_queues(void)
 {
 	int i;
 
-	quic_accept_queues = calloc(global.nbthread, sizeof(struct quic_accept_queue));
+	quic_accept_queues = calloc(global.nbthread,
+				    sizeof(*quic_accept_queues));
 	if (!quic_accept_queues) {
 		ha_alert("Failed to allocate the quic accept queues.\n");
 		return 0;
